-- TBLK TRLSE
drop table jf_munka;
drop table jf_projekt;
drop table jf_ceg;

-- SZMSOROK TRLSE
drop sequence jf_ceg_id_seq;
drop sequence jf_projekt_id_seq;
drop sequence jf_munka_id_seq;

-- TBLK LTREHOZSA
create table jf_ceg
  (id             number(3)     primary key not null,
   nev            varchar2(48)              not null,
   irsz           number(4),
   varos          varchar2(32),
   cim            varchar2(48),
   kapcsolattarto varchar2(32),
   email          varchar2(48),
   tel            varchar2(24),
   web            varchar2(32),
constraint jf_ceg_id check (id > 0));

create table jf_projekt
  (id                 number(3)     primary key       not null,
   megnevezes         varchar2(128)                   not null,
   ceg_id             number(3)                       not null,
   oradij             number(5)     default (2000)    not null,
   kiszallasi_dij     number(5)     default (1000),
   t_munkaora         number(3),
   kezdes             date          default (sysdate) not null,
   befejezes          date,
   fizetve            varchar2(1)   default ('N')     not null,
   bevetel            number(7)     default (0)       not null,
constraint jf_projekt_fkey foreign key (ceg_id) references jf_ceg (id) on delete cascade,
constraint jf_projekt_id             check (id > 0),
constraint jf_projekt_oradij         check (oradij >= 0),
constraint jf_projekt_kiszallasi_dij check (kiszallasi_dij >= 0),
constraint jf_projekt_t_munkaora     check (t_munkaora > 0),
constraint jf_projekt_befejezes      check (befejezes >= kezdes),
constraint jf_projekt_fizetve        check (fizetve in ('I', 'N')),
constraint jf_projekt_bevetel        check (bevetel >= 0));

create table jf_munka
  (id           number(3)      primary key                           not null,
   projekt_id   number(3)                                            not null,
   datum        date           default (sysdate)                     not null,
   kezdes       varchar2(5)    default (to_char(sysdate, 'HH24:MM')) not null,
   befejezes    varchar2(5)                                          not null,
   helyszini    varchar2(1)    default ('I')                         not null,
   leiras       varchar2(255),
constraint jf_munka_fkey foreign key (projekt_id) references jf_projekt (id) on delete cascade,
constraint jf_munka_id        check (id > 0),
constraint jf_munka_kezdes    check (kezdes like '__:__'),
constraint jf_munka_befejezes check (kezdes like '__:__'),
constraint jf_munka_helyszini check (helyszini in ('I', 'N')));

-- SZMSOROK LTREHOZSA
create sequence jf_ceg_id_seq 
  start with 1 
  increment by 1 
  nomaxvalue; 

create sequence jf_projekt_id_seq 
  start with 1 
  increment by 1 
  nomaxvalue; 

create sequence jf_munka_id_seq 
  start with 1 
  increment by 1 
  nomaxvalue;

-- TRIGGEREK LTREHOZSA
create or replace trigger jf_ceg_trg
  before insert on jf_ceg
  for each row
begin
  select jf_ceg_id_seq.nextval into :new.id from dual;
end;
/
create or replace trigger jf_projekt_trg
  before insert on jf_projekt
  for each row
begin
  select jf_projekt_id_seq.nextval into :new.id from dual;
end;
/
create or replace trigger jf_munka_trg
  before insert on jf_munka
  for each row
begin
  select jf_munka_id_seq.nextval into :new.id from dual;
end;
/
create or replace trigger jf_munka_iud_trg
  before insert or update or delete on jf_munka
  referencing old as old_buf new as new_buf 
  for each row
declare
  pid      number(3);      -- projekt_id
  k        number(1) := 0; -- volt-e kiszlls
  od       number(5);      -- radj
  kd       number(5);      -- kiszllsi dj
  munka    varchar2(200) := '';
  mp       number(5) := 0; -- ledolgozott percek
  ber      number(7) := 0; -- A projektben rgztett br
  regi_ber number(7) := 0; -- Mdosts sorn veszem hasznt
  uj_ber   number(7) := 0; -- Az j br

begin
  if inserting then
    pid := :new_buf.projekt_id;
  else
    pid := :old_buf.projekt_id;
  end if;

  select oradij, nvl(kiszallasi_dij, 0), nvl(bevetel, 0)
    into od, kd, ber
    from jf_projekt
    where id = pid;

  if inserting then
    munka := :new_buf.kezdes || '-' || :new_buf.befejezes;
    mp := MunkaidoI(munka);
    if :new_buf.helyszini = 'I' then
      k := 1;
    end if;
    uj_ber := ber + Berszamitas(mp, od, kd, k);
  end if;
  if deleting then
    munka := :old_buf.kezdes || '-' || :old_buf.befejezes;
    mp := MunkaidoI(munka);
    if :old_buf.helyszini = 'I' then
      k := 1;
    end if;
    uj_ber := ber - Berszamitas(mp, od, kd, k);
  end if;
  if updating then
    munka := :old_buf.kezdes || '-' || :old_buf.befejezes;
    mp := MunkaidoI(munka);
    if :old_buf.helyszini = 'I' then
      k := 1;
    end if;
    regi_ber := Berszamitas(mp, od, kd, k);

    munka := :new_buf.kezdes || '-' || :new_buf.befejezes;
    mp := MunkaidoI(munka);
    if :new_buf.helyszini = 'N' then
      k := 0;
    end if;
    uj_ber := ber - regi_ber + Berszamitas(mp, od, kd, k);
  end if;
  
  munka := 'update jf_projekt set bevetel = ' || uj_ber || ' where id = ' || pid;
  execute immediate munka;
end;